@@ -33,6 +33,10 @@ from werkzeug.datastructures import CombinedMultiDict, MultiDict
 import logging
 log = logging.getLogger("fsa")
 
+# module version…
+VERSION = "4.3.0dev"
+VERSION_NUM = (4, 3, -1)
+
 
 class AuthException(BaseException):
     """Exception class to carry fields for an error Response."""
@@ -72,7 +76,7 @@ class string(str):
     pass
 
 
-CASTS: Dict[type, Callable[[str], object]] = {
+_CASTS: Dict[type, Callable[[str], object]] = {
     bool: bool_cast,
     int: int_cast,
     # NOTE mypy complains wrongly about non-existing _empty.
@@ -84,6 +88,14 @@ CASTS: Dict[type, Callable[[str], object]] = {
     dt.datetime: dt.datetime.fromisoformat
 }
 
+
+def register_cast(t: type, c: Callable[[str], object]):
+    """Add a cast for a custom type, if the type itself does not work."""
+    if t in _CASTS:
+        log.warning(f"overriding type casting function for {t}")
+    _CASTS[t] = c
+
+
 #
 # SPECIAL PREDEFINED GROUP NAMES
 #
@@ -216,9 +228,11 @@ class Flask(flask.Flask):
 
     - the `route` decorator manages authentication, authorization and
       parameters transparently.
+    - per-methods shortcut decorators allow to handle root for a given
+      method: `get`, `post`, `put`, `patch`, `delete`.
     - several additional methods are provided: `get_user_pass`,
       `user_in_group`, `check_password`, `hash_password`, `create_token`,
-      `get_user`, `current_user`, `clear_caches`.
+      `get_user`, `current_user`, `clear_caches`, `register_cast`.
     """
 
     def __init__(self, *args, **kwargs):
@@ -241,6 +255,10 @@ class Flask(flask.Flask):
         """Set `user_in_group` helper function."""
         return self._fsa.user_in_group(uig)
 
+    def register_cast(self, t, c):
+        """Register a cast function for a type."""
+        self._fsa.register_cast(t, c)
+
     # password management
     def check_password(self, pwd, ref):
         """Check whether password is ok wrt to current password manager."""
@@ -288,7 +306,7 @@ _DIRECTIVES = {
     "FSA_MODE", "FSA_PARAM_PASS", "FSA_PARAM_USER", "FSA_PASSWORD_OPTIONS",
     "FSA_PASSWORD_SCHEME", "FSA_SKIP_PATH", "FSA_TOKEN_ALGO",
     "FSA_TOKEN_CARRIER", "FSA_TOKEN_DELAY", "FSA_TOKEN_GRACE",
-    "FSA_TOKEN_LENGTH", "FSA_TOKEN_NAME", "FSA_TOKEN_REALM",
+    "FSA_TOKEN_LENGTH", "FSA_TOKEN_NAME", "FSA_REALM",
     "FSA_TOKEN_SECRET", "FSA_TOKEN_SIGN", "FSA_TOKEN_TYPE",
     "FSA_URL_NAME", "FSA_USER_IN_GROUP", "FSA_LOGGING_LEVEL",
 }
@@ -434,6 +452,10 @@ class FlaskSimpleAuth:
         self._user_in_group = self._cache_function(uig)
         return uig
 
+    def register_cast(self, t, c):
+        """Register a cast function for a type."""
+        register_cast(t, c)
+
     #
     # DEFERRED INITIALIZATIONS
     #
@@ -508,7 +530,7 @@ class FlaskSimpleAuth:
         if need_carrier and not self._name:
             raise Exception(f"Token carrier {self._carrier} requires a name")
         # token realm…
-        realm = conf.get("FSA_TOKEN_REALM", self._app.name)
+        realm = conf.get("FSA_REALM", self._app.name)
         if self._token == "fsa":
             # simplify realm for fsa
             keep_char = re.compile(r"[-A-Za-z0-9]").match
@@ -814,7 +836,7 @@ class FlaskSimpleAuth:
     # FSA_TOKEN_SIGN: secret for signing new tokens for jwt pubkey algorithms
     # FSA_TOKEN_DELAY: token validity in minutes (60)
     # FSA_TOKEN_GRACE: grace delay for token validity in minutes (0)
-    # FSA_TOKEN_REALM: token realm (lc simplified app name)
+    # FSA_REALM: realm (lc simplified app name)
     #
     def _cmp_sig(self, data, secret):
         """Compute signature for data."""
@@ -1110,11 +1132,12 @@ class FlaskSimpleAuth:
 
         def decorate(fun: Callable):
 
+            # for each parameter name, its expected type, cast and default value
             types: Dict[str, type] = {}
             typings: Dict[str, Callable[[str], Any]] = {}
             defaults: Dict[str, Any] = {}
 
-            # parameters types/casts and defaults from signature
+            # parameters types/casts and defaults taken from signature
             sig = inspect.signature(fun)
 
             for n, p in sig.parameters.items():
@@ -1123,7 +1146,7 @@ class FlaskSimpleAuth:
                     # guess parameter type
                     t = typeof(p)
                     types[n] = t
-                    typings[n] = CASTS.get(t, t)
+                    typings[n] = _CASTS.get(t, t)
                 if p.default != inspect._empty:  # type: ignore
                     defaults[n] = p.default
 
@@ -1138,21 +1161,23 @@ class FlaskSimpleAuth:
                 params = self._params()
 
                 for p, typing in typings.items():
+                    # parameter HTTP name
+                    pn = p[1:] if p[0] == '_' else p
                     # guess which function parameters are request parameters
                     if p not in kwargs:
-                        if p in params:
+                        if pn in params:
                             try:
-                                kwargs[p] = typing(params[p])
+                                kwargs[p] = typing(params[pn])
                             except Exception as e:
-                                return self._Resp(f"type error on HTTP parameter \"{p}\" ({e})", 400)
+                                return self._Resp(f"type error on HTTP parameter \"{pn}\" ({e})", 400)
                         else:
                             if not required:
                                 if p in defaults:
                                     kwargs[p] = defaults[p]
                                 else:
-                                    return self._Resp(f"missing HTTP parameter \"{p}\"", 400)
+                                    return self._Resp(f"missing HTTP parameter \"{pn}\"", 400)
                             elif required:
-                                return f"missing HTTP parameter \"{p}\"", 400
+                                return f"missing HTTP parameter \"{pn}\"", 400
                             else:
                                 kwargs[p] = defaults.get(p, None)
                     else:
@@ -1161,7 +1186,7 @@ class FlaskSimpleAuth:
                             try:
                                 kwargs[p] = typing(kwargs[p])
                             except Exception as e:
-                                return self._Resp(f"type error on path parameter \"{p}\": ({e})", 404)
+                                return self._Resp(f"type error on path parameter \"{pn}\": ({e})", 404)
 
                 # possibly add others, without shadowing already provided ones
                 if allparams:
@@ -1176,10 +1201,12 @@ class FlaskSimpleAuth:
 
         return decorate
 
-    def add_url_rule(self, rule, endpoint=None, view_func=None, authorize=NONE, auth=None, required=None, allparams=False, **options):
-
+    def add_url_rule(self, rule, endpoint=None, view_func=None, authorize=NONE,
+                     auth=None, required=None, allparams=False, **options):
         """Route decorator helper method."""
 
+        log.debug(f"adding {rule}")
+
         # lazy initialization
         if not self._initialized:
             self.initialize()
@@ -1229,21 +1256,31 @@ class FlaskSimpleAuth:
     # support Flask 2.0 per-method decorator shortcuts
     # note that app.get("/", methods=["POST"], ...) would do a POST.
     def get(self, rule, **options):
+        """Shortcut for `route` with `GET` method."""
         return self.route(rule, methods=["GET"], **options)
 
     def post(self, rule, **options):
+        """Shortcut for `route` with `POST` method."""
         return self.route(rule, methods=["POST"], **options)
 
     def put(self, rule, **options):
+        """Shortcut for `route` with `PUT` method."""
         return self.route(rule, methods=["PUT"], **options)
 
     def delete(self, rule, **options):
+        """Shortcut for `route` with `DELETE` method."""
         return self.route(rule, methods=["DELETE"], **options)
 
     def patch(self, rule, **options):
+        """Shortcut for `route` with `PATCH` method."""
         return self.route(rule, methods=["PATCH"], **options)
 
     # duck-typing blueprint code stealing: needs blueprints and some other attributes.
     def register_blueprint(self, blueprint, **options):
         """Register a blueprint."""
+
+        # lazy initialization
+        if not self._initialized:
+            self.initialize()
+
         flask.Flask.register_blueprint(self, blueprint, **options)
